@@ -14,6 +14,22 @@ from textual.widget import Widget
 from textual.widgets import Input, Label
 
 
+async def test_query_errors():
+    app = App()
+    async with app.run_test():
+        with pytest.raises(InvalidQueryFormat):
+            app.query_one("foo_bar")
+
+        with pytest.raises(InvalidQueryFormat):
+            app.query("foo_bar")
+
+        with pytest.raises(InvalidQueryFormat):
+            app.query("1")
+
+        with pytest.raises(InvalidQueryFormat):
+            app.query_one("1")
+
+
 def test_query():
     class View(Widget):
         pass
@@ -103,7 +119,7 @@ def test_query():
         assert app.query_one("#widget1") == widget1
         assert app.query_one("#widget1", Widget) == widget1
         with pytest.raises(TooManyMatches):
-            _ = app.query_one(Widget)
+            _ = app.query_exactly_one(Widget)
 
         assert app.query("Widget.float")[0] == sidebar
         assert app.query("Widget.float")[0:2] == [sidebar, tooltip]
